#include "huffman.h"

huffman::huffman (string in_file_name, string out_file_name)
{
    (*this).in_file_name = in_file_name;
    in_file.open (in_file_name.c_str(), ios::in);
    out_file.open (out_file_name.c_str(), ios::out);
} // constructor


void huffman::create_pq()
{
    const string DEFAULT_TOTAL =
"With a fixed number of bits per character, the message size in bits is ";

    node_ptr entry;

    int sum = 0,
        n_chars = 0,
        n_bits_per_char;

    for (int i = 0; i < MAX_SIZE; i++)
    {
        node_array [i] = new huffman_node;
        (*node_array [i]).freq = 0;
    } // creating array

    create_node_array();

    for (int i = 0; i < MAX_SIZE; i++)
    {
        entry = node_array [i];
        if ((*entry).freq > 0)
        {
            pq.push (entry);
            sum += (*entry).freq;
            n_chars++;
        } // if
    } // counting chars and frequencies

    n_bits_per_char = ceil(log (n_chars) / log (2));
    cout << endl << DEFAULT_TOTAL << (sum * n_bits_per_char)<< endl;
} // create_pq


void huffman::create_node_array()
{
    node_ptr entry;

    string line;

    while (getline (in_file, line))
    {   
        for (unsigned j = 0; j < line.length(); j++)
        {
            entry = node_array [(int)(line [j])];
            (*entry).freq++;
            if ((*entry).freq == 1) 
            {
                (*entry).left = NULL;
                (*entry).right = NULL;
                (*entry).parent = NULL;
            } // first time character occurred
        } // for
        entry = node_array [(int)'\n'];
        (*entry).freq++;
        (*entry).left = NULL;
        (*entry).right = NULL;
        (*entry).parent = NULL;
    } // while
} // create_node_array


void huffman::create_huffman_tree()
{
    node_ptr left,
             right,
             sum;

    while( pq.size() > 1 )
    {
        left = pq.top();
        pq.pop();
        (*left).code = string( "0" );

        right = pq.top();
        pq.pop();
        (*right).code  = string( "1" );

        sum = new huffman_node;
        (*sum).parent  = NULL;
        (*sum).freq  = (*left).freq  + (*right).freq ;
        (*sum).left  = left;
        (*sum).right  = right;
        (*left).parent  = sum;
        (*right).parent  = sum;

        pq.push( sum );
    } // while
} // create_huffman_tree


void huffman::calculate_huffman_codes()
{
    const string HUFFMAN_CODES = "Here are the Huffman codes: ";

    const string ENCODED_SIZE_MESSAGE =
        "\n\nThe size of the encoded message, in bits, is ";

    int total = 0;

    string code;
    node_ptr entry;

    cout << endl << HUFFMAN_CODES << endl;
    for (int i = 0; i < MAX_SIZE; i++ )
    {
        code = "";
        entry = node_array [i];
        if ((*entry).freq > 0)
        {
            cout << (char)i  << "   ";
            do
            {
                code = (*entry).code + code;
                entry = (*entry).parent;
            } // do
            while ((*entry).parent != NULL);

            cout << code << endl;
            (*node_array [i]).code = code;
            total += code.length() * (*node_array [i]).freq ;
        } // if
    } // for
    cout << ENCODED_SIZE_MESSAGE << total << endl;
} // calculate_huffman_codes


void huffman::save_to_file()
{
    node_ptr entry;

    string line;

    in_file.close();
    in_file.open (in_file_name.c_str(), ios::in);

    for (int i = 0; i < MAX_SIZE; i++)
    {
        entry = node_array [i];
        if ((*entry).freq > 0)
            out_file << (char)i << " " << (*entry).code << endl;
    } // for
    out_file << "**" << endl;

    while (getline (in_file, line))
    {
        for (unsigned j = 0; j < line.length(); j++)
        {
            entry = node_array [(int)(line [j])];
            out_file << (*entry).code;
        } // for
        entry = node_array [(int)'\n'];
        out_file << (*entry).code;
    } // while
    out_file.close();
} // save_to_file



