#ifndef HUFFMAN
#define HUFFMAN

#include <fstream>
#include <math.h>       // declares log
#include <queue>
#include <vector>
#include <string>
#include <algorithm>
#include "huffman_node.h"

using namespace std;

const int MAX_SIZE = 256;

class huffman
{
   protected:

      node_ptr node_array [MAX_SIZE];
      fstream in_file,
              out_file;

      node_ptr child,
               parent,
               root;

      char id;

      string in_file_name;

      class compare
      {
         public:

            bool operator() ( const node_ptr& c1,
                              const node_ptr& c2 ) const
            { return (*c1).freq > (*c2).freq; }
      }; // compare

      priority_queue< node_ptr, vector<node_ptr>, compare > pq;

      // Postcondition: The node_array, including the frequency of each
      //                character, has been created from the input file.
      void create_node_array();


    public:

        // Postcondition: this huffman object has been initialized
        //                from in_file_name and out_file_name.
        huffman (string in_file_name, string out_file_name);


        // Postcondition: the priority queue has been created.
        void create_pq();

        // Postcondition: the Huffman tree has been created.
        void create_huffman_tree();


        // Postcondition: the Huffman codes have been calculated.
        void calculate_huffman_codes();


        // Postcondition: the Huffman codes and encoded message have been
        //                saved to a file.
        void save_to_file();

}; // class huffman

#endif

