#include <ctype.h>      // declares tolower(), isalpha()
#include <algorithm>    // defines most of the generic algorithms in STL
#include "spellcheck.h" // declares SpellCheck class

using namespace std;


void SpellCheck::readDictionaryFile()
{
   const string DICTIONARY_FILE = "dictfile.dat";

   fstream dictionaryFile;
   string word;

   dictionaryFile.open (DICTIONARY_FILE.c_str(), ios::in);

   while (dictionaryFile >> word) 
      dictionary.insert (word);
} // readDictionaryFile


void SpellCheck::readDocumentFile()
{
   const string DOCUMENT_FILE_PROMPT =
         "Please enter the name of the document file: ";

   const string DOCUMENT_MESSAGE = "Here is the document file:";
   fstream documentFile;
   string documentFileName,
          word;

   cout << endl << DOCUMENT_FILE_PROMPT;
   cin >> documentFileName;
   documentFile.open (documentFileName.c_str(), ios::in);

   cout << endl << endl << DOCUMENT_MESSAGE << endl;
   while (documentFile >> word)
   {
      cout << word << " ";

      // Make word lower case:
      string temp;
      for (unsigned i = 0; i < word.length(); i++)
          temp += tolower(word [i]);
      word = temp;

      // Eliminate punctuation at end of word:
      if (!isalpha (word[ word.length() - 1 ]))
         word = word.substr( 0, word.length() - 1 );

      // Insert word into words unless word is already there:
      if (words.find (word) == words.end()) {

         words.insert (word);
      } // if word not already in words
   } // while
   cout << endl;
} // readDocumentFile



void SpellCheck::compare()
{
   const string MISSPELLED =
         "Here are the possibly misspelled words:";

   AVLTree<string, less<string> >::Iterator itr;
   cout << endl << endl << endl << MISSPELLED << endl;
   for (itr = words.begin(); itr != words.end(); itr++)
      if (dictionary.find (*itr) == dictionary.end())
         cout << *itr << endl;
} // compare

int main()
{
    const string CLOSE_WINDOW_PROMPT =
      "Please press the Enter key to close this output window.";

    SpellCheck checker;

    checker.readDictionaryFile();
    checker.readDocumentFile();
    checker.compare();

    cout << endl << endl << CLOSE_WINDOW_PROMPT;
    cin.get();
    return 0;
}

